@@ -8,10 +8,10 @@ import math
 pd.options.mode.chained_assignment = None
 
 class TaSignals:
-    def __init__(self):
-        self.data = []
+    def __init__(self, window):
+        self.window = window
 
-    def on_balance_volume(self, df, n, key, window, front_run):
+    def on_balance_volume(self, df, n, key):
         """Calculate On-Balance Volume for given data.
 
         :param df: pandas.DataFrame
@@ -36,18 +36,29 @@ class TaSignals:
                 }
         return data, df
 
-    def bollinger(self, df, key, window, front_run):
+    def volume(self, df, key):
+        df['volume_slope'] = df['volume'].rolling(window=self.window).apply(self.get_slope, raw=True)
+        data = [
+            {'key': 'volume_rising', 'value': 'Volume is rising' if df['volume_slope'].iloc[-1] > 0 and df['volume_slope'].iloc[-1] < 10 else False},
+            {'key': 'volume_rising_fast', 'value': 'Volume is rising fast' if df['volume_slope'].iloc[-1] > 10 else False},
+            {'key': 'volume_dropping', 'value': 'Volume is dropping' if df['volume_slope'].iloc[-1] < 0 and df['volume_slope'].iloc[-1] > 10 else False},
+            {'key': 'volume_dropping_fast', 'value': 'Volume is dropping fast' if df['volume_slope'].iloc[-1] < -10  else False},
+        ]
+
+        return {}, df
+
+    def bollinger(self, df, key):
 
         tp = (df[key] + df['low'] + df['high'])/3
         df['std'] = tp.rolling(20).std(ddof=0)
-        df['matp'] = tp.rolling(20).mean()
-        df['bollinger_high'] = df['matp'] + 2*df['std']
-        df['bollinger_low'] = df['matp'] - 2*df['std']
+        matp = tp.rolling(20).mean()
+        df['bollinger_high'] = matp + 2*df['std']
+        df['bollinger_low'] = matp - 2*df['std']
         data = {'bollinger_low': df['bollinger_low'].iloc[-1], 'bollinger_high': df['bollinger_high'].iloc[-1]}
         data = [
-            {'key': 'bollinger_above_matp', 'value': 'Above bollinger matp' if (df[key].iloc[-1] > df['matp'].iloc[-1] + 25) else False},
-            {'key': 'bollinger_below_matp', 'value': 'Below bollinger matp' if (df[key].iloc[-1] < df['matp'].iloc[-1] - 25) else False},
-            {'key': 'bollinger_at_matp', 'value': 'At bollinger matp' if math.isclose(df[key].iloc[-1], df['matp'].iloc[-1], abs_tol=25) else False},
+            {'key': 'bollinger_above_matp', 'value': 'Above bollinger matp' if (df[key].iloc[-1] > matp.iloc[-1] + 25) else False},
+            {'key': 'bollinger_below_matp', 'value': 'Below bollinger matp' if (df[key].iloc[-1] < matp.iloc[-1] - 25) else False},
+            {'key': 'bollinger_at_matp', 'value': 'At bollinger matp' if math.isclose(df[key].iloc[-1], matp.iloc[-1], abs_tol=25) else False},
             {'key': 'below_bollinger_low', 'value': 'Below bollinger Low' if df[key].iloc[-1] < (df['bollinger_low'].iloc[-1] - 25) else False},
             {'key': 'above_bollinger_high', 'value': 'Above bollinger high' if df[key].iloc[-1] > (df['bollinger_high'].iloc[-1] + 25) else False},
             {'key': 'at_bollinger_low', 'value': 'At Bollinger low' if math.isclose(df[key].iloc[-1], df['bollinger_low'].iloc[-1], abs_tol=25) else False},
@@ -55,7 +66,7 @@ class TaSignals:
         ]
         return data, df
 
-    def ma(self, df, key, window, front_run):
+    def ma(self, df, key):
         df['ma20'] = df[key].rolling(20).mean()
         df['ma50'] = df[key].rolling(50).mean()
         df['ma100'] = df[key].rolling(100).mean()
@@ -75,7 +86,7 @@ class TaSignals:
         return data, df
 
 
-    def ema(self, df, key, window, front_run):
+    def ema(self, df, key):
         df['ema20'] = df[key].ewm(span=20, adjust=False).mean()
         df['ema50'] = df[key].ewm(span=50, adjust=False).mean()
         df['ema100'] = df[key].ewm(span=100, adjust=False).mean()
@@ -94,13 +105,15 @@ class TaSignals:
         ]
         return data, df
 
-    def rsi(self, df, key, window, front_run):
+    def rsi(self, df, key):
         df = self.get_rsi(df, key)
-        df['rsi_slope'] = df['rsi'].rolling(window=window).apply(self.get_slope, raw=True)
+        df['rsi_slope'] = df['rsi'].rolling(window=self.window).apply(self.get_slope, raw=True)
         data = {'rsi': df['rsi'].iloc[-1]}
         data = [
-            {'key': 'rsi_rising', 'value': 'RSI is rising' if df['rsi_slope'].iloc[-1] > 0 else False},
-            {'key': 'rsi_dropping', 'value': 'RSI is dropping' if df['rsi_slope'].iloc[-1]< 0 else False},
+            {'key': 'rsi_rising', 'value': 'RSI is rising' if df['rsi_slope'].iloc[-1] > 0 and df['rsi_slope'].iloc[-1] < 10 else False},
+            {'key': 'rsi_rising_fast', 'value': 'RSI is rising fast' if df['rsi_slope'].iloc[-1] > 10 else False},
+            {'key': 'rsi_dropping', 'value': 'RSI is dropping' if df['rsi_slope'].iloc[-1] < 0 and df['rsi_slope'].iloc[-1] > 10 else False},
+            {'key': 'rsi_dropping_fast', 'value': 'RSI is dropping fast' if df['rsi_slope'].iloc[-1] < -10  else False},
             {'key': 'rsi_oversold', 'value': 'RSI is oversold' if df['rsi'].iloc[-1] <= 30 and df['rsi'].iloc[-1] >= 20 else False},
             {'key': 'rsi_extremely_oversold', 'value': 'RSI is extremely oversold' if df['rsi'].iloc[-1] <= 20 else False},
             {'key': 'rsi_overbought', 'value': 'RSI is overbought' if df['rsi'].iloc[-1] >= 70  and df['rsi'].iloc[-1] <= 80 else False},
@@ -108,7 +121,7 @@ class TaSignals:
         ]
         return data, df
 
-    def macd_slope(self, df, key, window, front_run):
+    def macd_slope(self, df, key):
         k = df[key].ewm(span=12, adjust=False, min_periods=12).mean()
         d = df[key].ewm(span=26, adjust=False, min_periods=26).mean()
         macd = k - d
@@ -118,15 +131,17 @@ class TaSignals:
         df['MACDs_12_26_9'] = df.index.map(macd_s)
         df['MACDh_12_26_9'] = df.index.map(macd_h)
         #df.ta.macd(close=key, fast=12, slow=26, signal=9, append=True)
-        df['macd_slope'] = df['MACD_12_26_9'].rolling(window=window).apply(self.get_slope, raw=True)
-        df['macd_sig_slope'] = df['MACDs_12_26_9'].rolling(window=window).apply(self.get_slope, raw=True)
-        df['macd_hist_slope'] = df['MACDh_12_26_9'].rolling(window=window).apply(self.get_slope, raw=True)
+        df['macd_slope'] = df['MACD_12_26_9'].rolling(window=self.window).apply(self.get_slope, raw=True)
+        df['macd_sig_slope'] = df['MACDs_12_26_9'].rolling(window=self.window).apply(self.get_slope, raw=True)
+        df['macd_hist_slope'] = df['MACDh_12_26_9'].rolling(window=self.window).apply(self.get_slope, raw=True)
         data = {'macd_slope': df['macd_slope'].iloc[-1], 'macd_signal': df['MACDs_12_26_9'].iloc[-1], 'macd_hist': df['MACDh_12_26_9'].iloc[-1]}
         bullish_cross = math.isclose(df['MACD_12_26_9'].iloc[-1], df['MACDs_12_26_9'].iloc[-1], abs_tol=25) and df['macd_slope'].iloc[-1] > 0
         bearish_cross = math.isclose(df['MACD_12_26_9'].iloc[-1], df['MACDs_12_26_9'].iloc[-1], abs_tol=25) and df['macd_slope'].iloc[-1] < 0
         data = [
-            {'key': 'macd_rising', 'value': 'MACD is rising' if df['macd_slope'].iloc[-1] >= 0 else False},
-            {'key': 'macd_dropping', 'value': 'MACD is dropping' if df['macd_slope'].iloc[-1] <= 0 else False},
+            {'key': 'macd_rising_fast', 'value': 'MACD is rising fast' if df['macd_slope'].iloc[-1] > 20 else False},
+            {'key': 'macd_rising', 'value': 'MACD is rising' if df['macd_slope'].iloc[-1] > 0  and df['macd_slope'].iloc[-1] < 20 else False},
+            {'key': 'macd_dropping', 'value': 'MACD is dropping' if df['macd_slope'].iloc[-1] < 0 and df['macd_slope'].iloc[-1] > -20 else False},
+            {'key': 'macd_dropping_fast', 'value': 'MACD is dropping fast' if df['macd_slope'].iloc[-1] < -20 else False},
             {'key': 'bullish_macd_cross', 'value': 'Bullish MACD Crossover soon' if bullish_cross else False},
             {'key': 'bearish_macd_cross', 'value': 'Bearish MACD Crossover soon' if bearish_cross else False},
             {'key': 'macd_over_signal', 'value': 'MACD over Signal' if df['MACD_12_26_9'].iloc[-1] > df['MACDs_12_26_9'].iloc[-1] else False},
@@ -136,36 +151,21 @@ class TaSignals:
         ]
         return data, df
 
-    def divergence(self, df, key, window, front_run):
-        close_low_slope, close_high_slope, rsi_low_slope, rsi_high_slope, df = self.rsi_close_div_peak_slopes(df, key, window)
-        bullish_regular = (close_low_slope < 0) and (rsi_low_slope > 0)
-        bearish_regular = (close_high_slope > 0) and (rsi_high_slope < 0)
-        bullish_hidden = (rsi_high_slope < 0) and math.isclose(close_low_slope, 0, abs_tol=1)
-        bearish_hidden = (close_high_slope < 0) and math.isclose(close_high_slope, 0, abs_tol=1)
-        data = {'close_low_slope': close_low_slope, 'close_high_slope': close_high_slope, 'rsi_low_slope': rsi_low_slope, 'rsi_high_slope': rsi_high_slope}
-        data = [
-            {'key': 'bullish_regular', 'value': 'Bullish divergence' if bullish_regular else False},
-            {'key': 'bullish_hidden', 'value': 'Hidden Bullish divergence' if bullish_hidden else False},
-            {'key': 'bearish_regular', 'value': 'Bearish divergence' if bearish_regular else False},
-            {'key': 'bearish_hidden', 'value': 'Hidden Bearish divergence' if bearish_hidden else False},
-        ]
-        return data, df
-
     def low_idx(self, df, key, order):
         return argrelextrema(df[key].values, np.less_equal, order=order)[0]
 
     def high_idx(self, df, key, order):
         return argrelextrema(df[key].values, np.greater_equal, order=order)[0]
 
-    def peak_slope(self, df, key, window=4):
+    def peak_slope(self, df, key):
         lows, highs = self.range_slopes(df, key, 21, 21)
-        lows[f'{key}_lows_slope'] = lows[key].rolling(window=window).apply(self.get_slope, raw=True)
-        highs[f'{key}_highs_slope'] = highs[key].rolling(window=window).apply(self.get_slope, raw=True)
+        lows[f'{key}_lows_slope'] = lows[key].rolling(window=4).apply(self.get_slope, raw=True)
+        highs[f'{key}_highs_slope'] = highs[key].rolling(window=4).apply(self.get_slope, raw=True)
         return lows[f'{key}_lows_slope'].iloc[-1], highs[f'{key}_highs_slope'].iloc[-1], df
 
-    def rsi_close_div_peak_slopes(self, df, key, window=4):
-        close_low_slope, close_high_slope, df = self.peak_slope(df, key, window)
-        rsi_low_slope, rsi_high_slope, df = self.peak_slope(df, 'rsi', window)
+    def rsi_close_div_peak_slopes(self, df, key):
+        close_low_slope, close_high_slope, df = self.peak_slope(df, key)
+        rsi_low_slope, rsi_high_slope, df = self.peak_slope(df, 'rsi')
         df.merge(df)
         return close_low_slope, close_high_slope, rsi_low_slope, rsi_high_slope, df
 
@@ -187,9 +187,24 @@ class TaSignals:
         low_idx, high_idx = self.high_low_idx(df, key, low_window, high_window)
         lows, highs = df.iloc[low_idx].copy(), df.iloc[high_idx].copy()
         lows[f'{key}_lows_slope'] = lows[key].rolling(window=5).apply(self.get_slope, raw=True)
-        highs[f'{key}_highs_slope'] = highs[key].rolling(window=2).apply(self.get_slope, raw=True)
+        highs[f'{key}_highs_slope'] = highs[key].rolling(window=self.window).apply(self.get_slope, raw=True)
         return lows, highs
 
+    def divergence(self, df, key):
+        close_low_slope, close_high_slope, rsi_low_slope, rsi_high_slope, df = self.rsi_close_div_peak_slopes(df, key)
+        bullish_regular = (close_low_slope < 0) and (rsi_low_slope > 0)
+        bearish_regular = (close_high_slope > 0) and (rsi_high_slope < 0)
+        bullish_hidden = (rsi_high_slope < 0) and math.isclose(close_low_slope, 0, abs_tol=1)
+        bearish_hidden = (close_high_slope < 0) and math.isclose(close_high_slope, 0, abs_tol=1)
+        data = {'close_low_slope': close_low_slope, 'close_high_slope': close_high_slope, 'rsi_low_slope': rsi_low_slope, 'rsi_high_slope': rsi_high_slope}
+        data = [
+            {'key': 'bullish_regular', 'value': 'Bullish divergence' if bullish_regular else False},
+            {'key': 'bullish_hidden', 'value': 'Hidden Bullish divergence' if bullish_hidden else False},
+            {'key': 'bearish_regular', 'value': 'Bearish divergence' if bearish_regular else False},
+            {'key': 'bearish_hidden', 'value': 'Hidden Bearish divergence' if bearish_hidden else False},
+        ]
+        return data, df
+
     def get_slope(self, array):
         y = np.array(array)
         x = np.arange(len(y))
